

;--- Brushed ESC V2 By Rolf R Bakke, May 2012. Braking with reverse power and adjustable dragbrake.

; ----// edited V3 version, by Jozef Sklenar aka "coro" Sep2012 to become 2in1,
; take two rx inputs (thru two diodes and small enough resistor to ground needed) 
; to drive 2 motors e.g. robot or tank,
; with mix between common throttle for both and steering.
; added failsafe to stop without signal. Skipped arming check because of troubles.
; working version, but hardcoded with a lot of chaos by trying to solve some troubles.
; CAN NOT WORK WELL ON SINGLE Rx input due to mixing with steering, but somehow moves 1st motor.



;**** SETTINGS ****

.equ	BrakeStrength = 20	;0 to 127

.equ	DeadBand = 15		;n/127 parts

.equ	ThrottleNeutral = 1500	;uS

;******************


.include "m8def.inc"

.include "macros.inc"

.def	t=r16

.def	Phase=r17

.def	PwmOut1=r18

.def	Throttle1=r19

.def	tisp=r20

.def	sregsave=r21

.def	resetflags=r22

.def	Counter=r23

.def	Throttle2=r24

.def	PwmOut2=r25

.org 0
	rjmp reset
	rjmp unused
	rjmp unused
	rjmp unused
	rjmp unused
	rjmp unused
	rjmp unused
	rjmp unused
	rjmp unused
	rjmp timer0overflow
	rjmp unused
	rjmp unused
	rjmp unused
	rjmp unused
	rjmp unused
	rjmp unused
	rjmp unused
	rjmp unused
	rjmp unused

unused:	reti

reset:	
	in resetflags, mcucsr
	clr t
	out mcucsr, t

	ldi t, low(ramend)
	out spl, t
	ldi t, high(ramend)
	out sph, t


	;--- Port setup ---

	;        76543210
	ldi t, 0b11001100
	out ddrc, t

	;        76543210
	ldi t, 0b11111111
	out ddrb, t

	;        76543210
	ldi t, 0b10111011
	out ddrd, t


	;--- Timer 1 setup ---

	;        76543210
	ldi t, 0b00000000	
	out tccr1a, t

	;        76543210
	ldi t, 0b00000010	;1 MHz
	out tccr1b, t


	;--- ISR setup ---

	;        76543210
	ldi t, 0b00000001	
	out tccr0, t

	;        76543210
	ldi t, 0b00000001	
	out timsk, t


	;--- ADC setup ---

	;        76543210
	ldi t, 0b01100001	
	out admux, t

	;        76543210
	ldi t, 0b11100111	
	out adcsra, t


	;--- Variables setup ---

	clr Throttle1
	clr Throttle2


	;--- Reset cause signalling ---

	clr zl		;wait
	rcall wms


	sbrs resetflags, 3	;watchdog reset
	rjmp re1

	ldx 500
	ldy 500
	rcall sound
	rjmp re4

re1:	sbrs resetflags, 2	;Brown out reset
	rjmp re2

	ldx 300
	ldy 900
	rcall sound
	ldi zl,0
	rcall wms

	ldx 300
	ldy 1100
	rcall sound
	rjmp re4

re2:	sbrs resetflags, 1	;external reset
	rjmp re3

	ldx 300
	ldy 1000
	rcall sound
	ldi zl,0
	rcall wms
	ldx 300
	ldy 900
	rcall sound
	rjmp re4

re3:	sbrs resetflags, 0	;power on reset
	rjmp re4
	ldx 500
	ldy 1000
	rcall sound
	rjmp re4

re4:	

	;--- arming ---

ar2:	ldi Counter, 0

	ldx 300
	ldy 1000
	rcall sound
	ldi zl,0
	rcall wms

ar1:	
	rcall GetPwm
	
	ldy ThrottleNeutral - 4 * DeadBand
	cp  xl, yl
	cpc xh, yh
	brlo ar2		;too low, reset counter

	ldy ThrottleNeutral + 4 * DeadBand
	cp  xl, yl
	cpc xh, yh
	brsh ar2		;too high, reset counter

	inc Counter		;increase counter
	cpi Counter, 20		;50 in a row?
	// brne ar1		;no, arming not OK
	// bridged due to "Counter" abuse as Failsafe.


	;--- Armed sound ---
	ldy 500			;in range, make short sound
	ldx 500
	rcall sound

	ldi Counter, 20
	ldy 2000
ar3:	ldx 20
	rcall sound
	sbiw y, 63
	dec Counter
	brne ar3

	clr zl
	rcall wms

	;--- Main loop -----------------------------------------------------------

	sei

/* ma1:	
	rcall GetPwm		;get input PWM value

	subi xl, low(ThrottleNeutral)	;subtract throttle neutral
	sbci xh, high(ThrottleNeutral)

	asr xh			;divide by 2
	ror xl

	ldy 127			;limit upper value
	cp  xl, yl
	cpc xh, yh
	brlt ma2
	ldx 127
ma2:
	ldy -127		;limit lower value
	cp  xl, yl
	cpc xh, yh
	brge ma3
	ldx -127
ma3:
	mov Throttle1, xl	;set throttle

ma1b:	rcall GetPwm		;get input PWM value

	subi xl, low(ThrottleNeutral)	;subtract throttle neutral
	sbci xh, high(ThrottleNeutral)

	asr xh			;divide by 2
	ror xl

	ldy 127			;limit upper value
	cp  xl, yl
	cpc xh, yh
	brlt ma2b
	ldx 127
ma2b:
	ldy -127		;limit lower value
	cp  xl, yl
	cpc xh, yh
	brge ma3b
	ldx -127
ma3b:
	mov Throttle2, xl	;set throttle
*/

ma1:	
	ldi  Counter,127
	rcall GetPwm		;get input PWM value

	subi xl, low(ThrottleNeutral)	;subtract throttle neutral
	sbci xh, high(ThrottleNeutral)

	asr xh			;divide by 2
	ror xl

	ldy 127			;limit upper value
	cp  xl, yl
	cpc xh, yh
	brlt ma2
	ldx 127
ma2:
	ldy -127		;limit lower value
	cp  xl, yl
	cpc xh, yh
	brge ma3
	ldx -127
ma3:
	mov Throttle1, xl	;set throttle

ma1b:	rcall GetPwm		;get input PWM value

	subi xl, low(ThrottleNeutral)	;subtract throttle neutral
	sbci xh, high(ThrottleNeutral)

	asr xh			;divide by 2
	ror xl

	ldy 127			;limit upper value
	cp  xl, yl
	cpc xh, yh
	brlt ma2b
	ldx 127
ma2b:
	ldy -127		;limit lower value
	cp  xl, yl
	cpc xh, yh
	brge ma3b
	ldx -127
ma3b:
	// not now - 
	mov Throttle2, xl	;set throttle2

//now a mix of power and steering out of thr1 and xl
// power = Thr1
// steering = Thr2
//thr1=Thr1+Thr2 - can overflow both positive and negative
//thr2=Thd1-Thr2
	mov xl, Throttle1
	clr yh
	mov yl, Throttle2
	add yl,xl
	brvc ma3c // ok 
	brpl plusc
	ldi yl,127
	rjmp ma3c
plusc:
	ldi yl,-127
ma3c:
	mov yh,Throttle2

	mov Throttle1,yl
	mov yl,yh
	clr yh
	sub yl,xl
	brvc ma3d //
	brpl plusd
	ldi yl,127
	rjmp ma3d
plusd:
	ldi yl,-127
ma3d:
	mov Throttle2,yl


ma5:	
	in t, adch		;high temp?
	cpi t, 30
	brsh ma4

	ldx 150			;yes, sound the alarm until it has cooled down
	ldy 1200
	rcall sound
	ldx 100
	ldy 2000
	rcall sound
	rjmp ma5
ma4:
	//wait at least 5ms to force first channel be throttle1
	ldi zl,5
	rcall wms
	

	rjmp ma1


	;--- SUBS ---
	

	;--- get PWM input value ---


GetPwm:
	;wait for low to high transition on ppm input

ge1:	sbic pind, 2	//high to low
	// ATTEMPT to SKIP  
	//rjmp ge1
ge2:	sbis pind, 2	//low to high
	rjmp ge2

	;start counting

	clr t
	out tcnt1h, t
	out tcnt1l, t

	;wait for low edge

ge3:	sbic pind, 2
	rjmp ge3

	;get time

	in xl, tcnt1l
	in xh, tcnt1h

	ret


	;--- ISR ---

timer0overflow:

	in sregsave, sreg

	ldi tisp, 0xb0		;reload ISR counter
	out tcnt0, tisp

;debug_on

	inc Phase		;PWM generator
	cpi Phase, 128
	brne tm1

;debug_on

	;this part is run once, right at the start of the PWM period. 

	clr Phase
	dec Counter
	brne newthrottle

	clr Throttle1 // no signal
	clr Throttle2

newthrottle:
	mov PwmOut2, Throttle2	;sample throttle2 value
	mov PwmOut1, Throttle1	;sample throttle1 value

	cpi PwmOut1, DeadBand	;brake or power?
	brge tm4
	cpi PwmOut1, -DeadBand
	brlt tm4

	cpi PwmOut2, DeadBand	;brake or power?
	brge tm4
	cpi PwmOut2, -DeadBand
	brlt tm4

	a_bottom_on		;Brake. Only if both off - No brake if any side should run.
	b_bottom_on
	c_bottom_on
	
	ldi PwmOut1, BrakeStrength
	ldi PwmOut2, BrakeStrength
	rjmp tm2

tm4: ; SOME kind of Power. Which direction? And which one?
	tst PwmOut1		
	brmi tm3_REV_1
	tst PwmOut2
	brmi tm3_REV_2


	;forward both

;debug_on

	a_top_on
	c_top_on
	nop		;delay to let top FET turn on before the bottom FET (NFET are better at switching under load)
	b_bottom_on

	rjmp tm2


	; left reverse, right forward. Ehm.. so.. 
	// which one is forward, let it run first but only half of the time(signed)
	// other stays negative  to see it later
	
tm3_REV_1: ;reverse 1
	tst PwmOut2
	brmi tm3_REV_BOTH

	//start half of the time 1 forward 
	asr PwmOut1 //div 2
	asr PwmOut2 //negative here
	c_top_on
	b_bottom_on
	rjmp tm2
	
tm3_REV_2: ;reverse 2
	asr PwmOut2
	asr PwmOut1
	//add PwmOut2,PwmOut1
	a_top_on
	b_bottom_on
	rjmp tm2

tm3_REV_BOTH: //only case when backward may run
	neg PwmOut1 
	neg PwmOut2 
	b_top_on
	nop		;delay to let top FET turn on before the bottom FET
	nop
	a_bottom_on
	c_bottom_on
	rjmp tm2


	; This part turns off the transistors when Phase is => PwmOut
	; AND PwmOut is not negative - if it is, start reverse part.

tm1:	
	cp Phase, PwmOut1 
	brlo tm1b

	a_bottom_off //timeout pwm1
	a_top_off
	tst PwmOut2
	brmi tm1restart2

tm1b:
	cp Phase, PwmOut2
	brlo tm2 // not yet
	   ;bottom off first to let the top switch without current
	c_bottom_off // timeout pwm1
	c_top_off
	tst PwmOut1
	brmi tm1restart1

	cp Phase, PwmOut1 //once again, is A off already too?
	brlo tm2
	b_bottom_off
	b_top_off
	rjmp tm2 // all done

tm1restart2:
	// Pwm1 is over, bur it still should start neg Pwm2 +phase.
	b_bottom_off
	c_top_off
	neg PwmOut2
	add PwmOut2, Phase
	b_top_on 
	nop
	c_bottom_on
	rjmp tm2
tm1restart1:
	// Pwm2 is over, bur it still should start neg Pwm1 +phase.
	b_bottom_off
	a_top_off
	neg PwmOut1
	add PwmOut1, Phase
	b_top_on 
	nop
	a_bottom_on
	rjmp tm2



	;exit

tm2:
;debug_off
	out sreg, sregsave
	reti



	;--- Sound generation ---

sound:	cli

	a_bottom_off
	c_bottom_off
	b_bottom_off
	a_top_off
	c_top_off
	b_top_off

	ldz 100
	rcall delay

so1:	a_top_on
	nop
	nop
	nop
	b_bottom_on

	ldz 200
	rcall delay
	
	a_bottom_off
	b_bottom_off
	a_top_off
	b_top_off

	movw z, y
	rcall delay

	b_top_on
	nop
	nop
	nop
	a_bottom_on

	ldz 200
	rcall delay
	
	a_bottom_off
	b_bottom_off
	a_top_off
	b_top_off

	movw z, y
	rcall delay

	sbiw x,1
	brne so1
	
	sei
	ret


	;--- Delay ---

delay:	sbiw z, 1
	brne delay
	ret

wms:	 ldi t,235	; wait zl ms
wm1:	  ldi zh,11	;34
wm2:	  dec zh
	  brne wm2
	 dec t
	 brne wm1
	dec zl
	brne wms
	ret




